/*
**    Projet etudiants CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef QUATERNION_HPP
#define QUATERNION_HPP

#include <v4.hpp>

namespace cpe
{
    class v3;
    class matrix3;

    /** \brief Quaternion class */
    class quaternion: public v4
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief default constructor */
        quaternion();
        /** \brief constructor from axis and angle */
        quaternion(const v3& axis,const double& angle);
        /** \brief direct constructor */
        quaternion(const v4& entries);


        // ********************************************* //
        // ********************************************* //
        //  GET
        // ********************************************* //
        // ********************************************* //

        /** \brief get the unit axis corresponding to the quaternion */
        v3 axis() const;
        /** \brief get the angle corresponding to the quaternion */
        double angle() const;


        // ********************************************* //
        // ********************************************* //
        //  Math
        // ********************************************* //
        // ********************************************* //


        /** \brief compute the rotation matrix associated to the current quaternion
            \return a 3x3 rotation matrix
        */
        matrix3 matrix() const;

        /** \brief get the conjugate of the current quaterion */
        quaternion conjugated() const;

        /** \brief Spherical Linear Interpolation SLERP

            Use (sin((1-t) theta )q0 +sin(t theta)q1)/sin(theta)
            But same results with (q1 q0^{-1})^{t} q0
            (q1*q0.invert()).pow_q(t)*q0) */
        static quaternion slerp(const quaternion& q0,const quaternion& q1,const double& t);


        // ********************************************* //
        // ********************************************* //
        //  Operators
        // ********************************************* //
        // ********************************************* //

        /** \brief multiplication between quaternion */
        friend quaternion operator*(const quaternion& q0,const quaternion& q1);
        /** \brief internal multiplication between quaternion */
        quaternion& operator*=(const quaternion& q1);

        /** \brief multiplication with a vector */
        friend v3 operator*(const quaternion& q,const v3& v);

    private:

    };
}

#endif
