
/*
**    Projet etudiants CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <scene.hpp>

#include <glwidget.hpp>
#include <cmath>




//constructeur de scene: initialisation des variables de la classe
scene::scene(glwidget *p_glwidget_)
    :p_glwidget(p_glwidget_)
{
    affiche_texture=true;
}

//chargement du model 3D: n'est execute qu'une fois en tout debut de programme
void scene::load_model()
{

    QPixmap pix("data/explosion.jpg");
    tex=p_glwidget->bindTexture(pix,GL_TEXTURE_2D);
    try
    {
        //chargement des objets geometriques
        objet_1=objet_affichable("data/armor.off");
        objet_2=objet_affichable("data/canon.off");
        objet_3=objet_affichable("data/chain.off");
    }
    catch(std::string e)
    {
        std::cout<<"Exception: "<<e<<std::endl;
    }

}


//affichage de la scene: executee au cours du programme a de nombreuses reprises
void scene::draw_scene()
{

    glEnable(GL_LIGHTING); //gestion de la lumiere


    //affichage des objets geometriques
    glColor3d(0.6,0.6,0.6);//couleur courante (r,b,g)
    objet_1.affiche(); //affichage objet_1

    glColor3d(0.6,0.65,0.6);//couleur courante (r,b,g)
    objet_2.affiche(); //affichage objet_2

    glColor3d(0.6,0.6,0.65);//couleur courante (r,b,g)
    objet_3.affiche(); //affichage objet_3


    //affiche la texture si on le souhaite
    glDisable(GL_LIGHTING);
    if(affiche_texture==true)
        affichage_texture_exemple();
    glEnable(GL_LIGHTING);

    //affiche un texte dans la scene en noir
    glColor3d(0,0,0);
    p_glwidget->renderText(0,-2,0,"Bienvenue dans mon jeu",QFont("Times",15,QFont::Bold));


}

//exemple d'affichage d'un objet texturee
void scene::affichage_texture_exemple() const
{

    //affiche un objet texturee entre les coordoonnees (0,0,0) et (1,1,0)

    glColor3d(1,1,1); //met la couleur courante a blanc
    glEnable(GL_TEXTURE_2D); //active l'utilisation de la texture
    glBindTexture(GL_TEXTURE_2D,tex); //utilisation de la texture identifiee par tex

    glBegin(GL_QUADS); //debute l'affichage d'un quadrangle
    glNormal3d(0,0,1); //la normale du quadrangle

    glTexCoord2d(0,0); //composante de texture
    glVertex3d(-2,-1,0); //coordonnee du sommet 1

    glTexCoord2d(1,0); //composante de texture
    glVertex3d(-1,-1,0); //coordonnee du sommet 2

    glTexCoord2d(1,1); //composante de texture
    glVertex3d(-1,-2,0); //coordonnee du sommet 2

    glTexCoord2d(0,1); //composante de texture
    glVertex3d(-2,-2,0); //coordonnee du sommet 2
    glEnd(); //fin du quadrangle

    glDisable(GL_TEXTURE_2D); //desactive l'utilisation des textures
}
void scene::set_affichage_texture(const bool& est_actif)
{
    affiche_texture=est_actif;
}

